import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/l10n/messages_all.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

class AppLocalizationsDelegate extends LocalizationsDelegate<AppLocalizations> {
  const AppLocalizationsDelegate();

  List<Locale> get supportedLocales {
    return const <Locale>[
      Locale('en'),
      Locale('es'),
      Locale('pt'),
    ];
  }

  @override
  bool isSupported(Locale locale) {
    for (var supportedLocale in supportedLocales) {
      if (supportedLocale.languageCode == locale.languageCode) {
        return true;
      }
    }
    return false;
  }

  @override
  Future<AppLocalizations> load(Locale locale) {
    return AppLocalizations.load(locale);
  }

  @override
  bool shouldReload(LocalizationsDelegate<AppLocalizations> old) {
    return false;
  }
}

class AppLocalizations {
  final Locale locale;

  AppLocalizations(this.locale);

  static const AppLocalizationsDelegate delegate = AppLocalizationsDelegate();

  static Future<AppLocalizations> load(Locale locale) {
    final name =
        locale.countryCode == null ? locale.languageCode : locale.toString();
    final localeName = Intl.canonicalizedLocale(name);

    return initializeMessages(localeName).then((bool _) {
      Intl.defaultLocale = localeName;
      return AppLocalizations(locale);
    });
  }

  static AppLocalizations? of(BuildContext context) {
    return Localizations.of<AppLocalizations>(context, AppLocalizations);
  }

  //To dynamically load a string by name
  String withName(String name) {
    return Intl.message(
      '',
      name: name,
    );
  }

  String get common_button_continue {
    return Intl.message(
      'Continue',
      name: 'common_button_continue',
    );
  }

  String get common_label_and {
    return Intl.message(
      'and',
      name: 'common_label_and',
    );
  }

  String get common_label_privacyPolicy {
    return Intl.message(
      'Privacy Policy',
      name: 'common_label_privacyPolicy',
    );
  }

  String get common_label_termsAndConditions {
    return Intl.message(
      'Terms & Conditions',
      name: 'common_label_termsAndConditions',
    );
  }

  String get common_label_version {
    return Intl.message(
      'Version',
      name: 'common_label_version',
    );
  }

  String get common_error_missingParameters {
    return Intl.message(
      'Missing parameters',
      name: 'common_error_missingParameters',
    );
  }

  String get common_error_default {
    return Intl.message(
      'An error has occurred',
      name: 'common_error_default',
    );
  }

  String get enrollmentFormScreen_label_agreeTo {
    return Intl.message(
      'I agree to',
      name: 'enrollmentFormScreen_label_agreeTo',
    );
  }

  String get enrollmentFormScreen_label_mandatoryDocuments {
    return Intl.message(
      'Please, confirm you agree to the following documents',
      name: 'enrollmentFormScreen_label_mandatoryDocuments',
    );
  }

  String get enrollmentFormScreen_label_updatedDocuments {
    return Intl.message(
      'The following documents have been updated and you need to accept the changes in order to continue',
      name: 'enrollmentFormScreen_label_updatedDocuments',
    );
  }

  String get enrollmentFormScreen_label_summaryOfChanges {
    return Intl.message(
      'Summary of changes',
      name: 'enrollmentFormScreen_label_summaryOfChanges',
    );
  }

  String get enrollmentFormScreen_button_viewDocument {
    return Intl.message(
      'View document',
      name: 'enrollmentFormScreen_button_viewDocument',
    );
  }
}
